@@ -27,33 +27,14 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include "asoundlib.h"
+#include "seq_priv.h"
 
 #define SND_FILE_SEQ		"/dev/snd/seq"
 #define SND_FILE_ALOADSEQ	"/dev/aloadSEQ"
-#define SND_SEQ_VERSION_MAX	SND_PROTOCOL_VERSION( 0, 0, 1 )
+#define SND_SEQ_VERSION_MAX	SND_PROTOCOL_VERSION(1, 0, 0)
 #define SND_SEQ_OBUF_SIZE	(16*1024)	/* should be configurable */
 #define SND_SEQ_IBUF_SIZE	(4*1024)	/* should be configurable */
 
-typedef struct snd_stru_seq_cell {
-	snd_seq_event_t ev;
-	struct snd_stru_seq_cell *next;
-} snd_seq_cell_t;
-
-struct snd_seq {
-	int client;		/* client number */
-	int fd;
-	/* buffers */
-	char *obuf;		/* output buffer */
-	int obufsize;		/* output buffer size */
-	int obufused;		/* output buffer used size */
-	char *ibuf;		/* input buffer */
-	int ibufsize;		/* input buffer size */
-	/* input queue */
-	int cells;
-	snd_seq_cell_t *head;
-	snd_seq_cell_t *tail;
-};
-
 int snd_seq_open(snd_seq_t **handle, int mode)
 {
 	int fd, ver, client, flg;
@@ -115,6 +96,10 @@ int snd_seq_close(snd_seq_t *seq)
 	if (!seq)
 		return -EINVAL;
 	res = close(seq->fd) < 0 ? -errno : 0;
+	if (seq->obuf)
+		free(seq->obuf);
+	if (seq->ibuf)
+		free(seq->ibuf);
 	free(seq);
 	return res;
 }
@@ -347,6 +332,7 @@ int snd_seq_set_queue_timer(snd_seq_t *seq, int q, snd_seq_queue_timer_t * timer
 	return 0;
 }
 
+#ifdef use_seqsync
 int snd_seq_get_queue_sync(snd_seq_t *seq, int q, snd_seq_queue_sync_t * sync)
 {
 	if (!seq || !sync)
@@ -367,6 +353,7 @@ int snd_seq_set_queue_sync(snd_seq_t *seq, int q, snd_seq_queue_sync_t * sync)
 		return -errno;
 	return 0;
 }
+#endif
 
 int snd_seq_get_queue_client(snd_seq_t *seq, int q, snd_seq_queue_client_t * info)
 {
@@ -391,48 +378,76 @@ int snd_seq_set_queue_client(snd_seq_t *seq, int q, snd_seq_queue_client_t * inf
 	return 0;
 }
 
-int snd_seq_alloc_queue(snd_seq_t *seq)
+int snd_seq_alloc_named_queue(snd_seq_t *seq, char *name)
 {
-	int i, err;
-	snd_seq_system_info_t sysinfo;
-	snd_seq_queue_owner_t owner;
+	snd_seq_queue_info_t info;
 
 	if (!seq)
 		return -EINVAL;	
-	if ((err = snd_seq_system_info(seq, &sysinfo))<0)
-		return err;
-	for (i = 0; i < sysinfo.queues; i++) {
-		if ((err = snd_seq_get_queue_owner(seq, i, &owner))<0)
-			continue;
-		if (owner.locked)
-			continue;
-		owner.locked = 1;
-		owner.owner = seq->client;
-		if ((err = snd_seq_set_queue_owner(seq, i, &owner))<0)
-			continue;
-		return i;
-	}
-	return -EBUSY;
+
+	memset(&info, 0, sizeof(info));
+	info.owner = seq->client;
+	info.locked = 1;
+	if (name)
+		strncpy(info.name, name, sizeof(info.name) - 1);
+	if (ioctl(seq->fd, SND_SEQ_IOCTL_CREATE_QUEUE, &info) < 0)
+		return -errno;
+
+	return info.queue;
+}
+
+int snd_seq_alloc_queue(snd_seq_t *seq)
+{
+	return snd_seq_alloc_named_queue(seq, NULL);
 }
 
 int snd_seq_free_queue(snd_seq_t *seq, int q)
 {
-	int err;
-	snd_seq_queue_owner_t owner;
+	snd_seq_queue_info_t info;
 
 	if (!seq)
 		return -EINVAL;	
-	if ((err = snd_seq_get_queue_owner(seq, q, &owner))<0)
-		return err;
-	if (owner.locked && owner.owner == seq->client) {
-		owner.locked = 0;
-		owner.owner = -1;
-		if ((err = snd_seq_set_queue_owner(seq, q, &owner))<0)
-			return err;
-	}
+
+	memset(&info, 0, sizeof(info));
+	info.queue = q;
+	if (ioctl(seq->fd, SND_SEQ_IOCTL_DELETE_QUEUE, &info) < 0)
+		return -errno;
+
 	return 0;
 }
 
+int snd_seq_get_queue_info(snd_seq_t *seq, int q, snd_seq_queue_info_t *info)
+{
+	if (!seq || !info)
+		return -EINVAL;
+	info->queue = q;
+	if (ioctl(seq->fd, SND_SEQ_IOCTL_GET_QUEUE_INFO, info) < 0)
+		return -errno;
+	return 0;
+}
+
+int snd_seq_set_queue_info(snd_seq_t *seq, int q, snd_seq_queue_info_t *info)
+{
+	if (!seq || !info)
+		return -EINVAL;
+	info->queue = q;
+	if (ioctl(seq->fd, SND_SEQ_IOCTL_SET_QUEUE_INFO, info) < 0)
+		return -errno;
+	return 0;
+}
+
+int snd_seq_get_named_queue(snd_seq_t *seq, char *name)
+{
+	snd_seq_queue_info_t info;
+
+	if (!seq)
+		return -EINVAL;
+	strncpy(info.name, name, sizeof(info.name));
+	if (ioctl(seq->fd, SND_SEQ_IOCTL_GET_NAMED_QUEUE, &info) < 0)
+		return -errno;
+	return info.queue;
+}
+
 snd_seq_event_t *snd_seq_create_event(void)
 {
 	return (snd_seq_event_t *) calloc(1, sizeof(snd_seq_event_t));
@@ -442,11 +457,9 @@ static int snd_seq_free_event_static(snd_seq_event_t *ev)
 {
 	if (!ev)
 		return -EINVAL;
-	switch (ev->flags & SND_SEQ_EVENT_LENGTH_MASK) {
-	case SND_SEQ_EVENT_LENGTH_VARIABLE:
+	if (snd_seq_ev_is_variable(ev)) {
 		if (ev->data.ext.ptr)
 			free(ev->data.ext.ptr);
-		break;
 	}
 	return 0;
 }
@@ -467,33 +480,37 @@ int snd_seq_event_length(snd_seq_event_t *ev)
 
 	if (!ev)
 		return -EINVAL;
-	switch (ev->flags & SND_SEQ_EVENT_LENGTH_MASK) {
-	case SND_SEQ_EVENT_LENGTH_VARIABLE:
+	if (snd_seq_ev_is_variable(ev)) {
+		if (ev->data.ext.len < 0)
+			return -EINVAL;
 		len += ev->data.ext.len;
-		break;
 	}
 	return len;
 }
 
 int snd_seq_event_output(snd_seq_t *seq, snd_seq_event_t *ev)
 {
-	int len;
+	int len, err;
 
 	if (!seq || !ev)
 		return -EINVAL;
 	len = snd_seq_event_length(ev);
+	if (len < 0)
+		return -EINVAL;
 	if ((seq->obufsize - seq->obufused) < len) {
-		snd_seq_flush_output(seq);
+		err = snd_seq_flush_output(seq);
+		if (err < 0)
+			return err;
 		if ((seq->obufsize - seq->obufused) < len)
 			return -ENOMEM;
 	}
 	memcpy(seq->obuf + seq->obufused, ev, sizeof(snd_seq_event_t));
 	seq->obufused += sizeof(snd_seq_event_t);
-	switch (ev->flags & SND_SEQ_EVENT_LENGTH_MASK) {
-	case SND_SEQ_EVENT_LENGTH_VARIABLE:
+	if (snd_seq_ev_is_variable(ev)) {
+		if (ev->data.ext.len < 0)
+			return -EINVAL;
 		memcpy(seq->obuf + seq->obufused, ev->data.ext.ptr, ev->data.ext.len);
 		seq->obufused += ev->data.ext.len;
-		break;
 	}
 	return seq->obufused;
 }
@@ -555,7 +572,7 @@ static int snd_seq_input_cell_available(snd_seq_t *seq)
 {
 	if (!seq)
 		return -EINVAL;
-	return seq->cells > 0;
+	return seq->cells;
 }
 
 static int snd_seq_decode_event(char **buf, int *len, snd_seq_event_t *ev)
@@ -569,24 +586,24 @@ static int snd_seq_decode_event(char **buf, int *len, snd_seq_event_t *ev)
 	memcpy(ev, *buf, sizeof(snd_seq_event_t));
 	*buf += sizeof(snd_seq_event_t);
 	*len -= sizeof(snd_seq_event_t);
-	switch (ev->flags & SND_SEQ_EVENT_LENGTH_MASK) {
-	case SND_SEQ_EVENT_LENGTH_VARIABLE:
+	if (snd_seq_ev_is_variable(ev)) {
 		if (*len < ev->data.ext.len) {
 			*len = 0;
+			ev->flags &= ~SND_SEQ_EVENT_LENGTH_MASK; /* clear flag */
 			return -ENOENT;
 		}
 		if (ev->data.ext.len > 0) {
 			ev->data.ext.ptr = (char *) malloc(ev->data.ext.len);
 			if (!(ev->data.ext.ptr)) {
+				ev->flags &= ~SND_SEQ_EVENT_LENGTH_MASK; /* clear flag */
 				*buf += ev->data.ext.len;
 				*len -= ev->data.ext.len;
-				return -ENOENT;
+				return -ENOMEM;
 			}
 			memcpy(ev->data.ext.ptr, *buf, ev->data.ext.len);
 			*buf += ev->data.ext.len;
 			*len -= ev->data.ext.len;
 		}
-		break;
 	}
 	return 0;
 }
@@ -595,68 +612,266 @@ static int snd_seq_decode_event(char **buf, int *len, snd_seq_event_t *ev)
  *  Current implementation uses FIFO cache.
  */
 
-int snd_seq_event_input(snd_seq_t *seq, snd_seq_event_t **ev)
+static int snd_seq_event_read_buffer(snd_seq_t *seq)
 {
-	snd_seq_cell_t *cell;
 	char *buf;
 	int count;
 
-	*ev = NULL;
-	if (!seq)
-		return -EINVAL;
-	if (snd_seq_input_cell_available(seq)) {
-		*ev = snd_seq_create_event();
-		if (*ev == NULL)
-			return -ENOMEM;
-		cell = snd_seq_input_cell_out(seq);
-		memcpy(*ev, &cell->ev, sizeof(snd_seq_event_t));
-		return seq->cells;
-	}
 	count = read(seq->fd, seq->ibuf, seq->ibufsize);
 	if (count < 0)
 		return -errno;
 	buf = seq->ibuf;
 	while (count > 0) {
-		if (*ev == NULL) {	/* first event */
-			*ev = snd_seq_create_event();
-			if (*ev == NULL)
-				return -ENOMEM;
-			if (snd_seq_decode_event(&buf, &count, *ev)<0) {
-				snd_seq_free_event(*ev);
-				*ev = NULL;
-			}
+		snd_seq_cell_t *cell = snd_seq_create_cell(NULL);
+		if (cell == NULL)
+			return -ENOMEM;
+		if (snd_seq_decode_event(&buf, &count, &cell->ev)<0) {
+			snd_seq_free_cell(cell);
 		} else {
-			cell = snd_seq_create_cell(NULL);
-			if (cell == NULL)
-				return -ENOMEM;
-			if (snd_seq_decode_event(&buf, &count, &cell->ev)<0) {
-				snd_seq_free_cell(cell);
-			} else {
-				snd_seq_input_cell_in(seq, cell);
-			}
+			snd_seq_input_cell_in(seq, cell);
 		}
 	}
 	return seq->cells;
 }
 
+/*
+ * retrieve an event from FIFO
+ * if fifo is empty, read events from sequencer
+ */
+int snd_seq_event_input(snd_seq_t *seq, snd_seq_event_t **ev)
+{
+	snd_seq_cell_t *cell;
+
+	*ev = NULL;
+	if (!seq)
+		return -EINVAL;
+
+	if (! snd_seq_input_cell_available(seq)) {
+		int err = snd_seq_event_read_buffer(seq);
+		if (err < 0)
+			return err;
+	}
+
+	if (snd_seq_input_cell_available(seq)) {
+		*ev = snd_seq_create_event();
+		if (*ev == NULL)
+			return -ENOMEM;
+		cell = snd_seq_input_cell_out(seq);
+		if (cell == NULL)
+			return -ENOMEM;
+		memcpy(*ev, &cell->ev, sizeof(snd_seq_event_t));
+		/* clear flag to avoid free copied data */
+		cell->ev.flags &= ~SND_SEQ_EVENT_LENGTH_MASK;
+		snd_seq_free_cell(cell);
+		return seq->cells;
+	}
+	return seq->cells;
+}
+
+/*
+ * read input data from sequencer if available
+ */
+static int snd_seq_event_input_feed(snd_seq_t *seq, struct timeval *timeout)
+{
+	fd_set rfds;
+
+	FD_ZERO(&rfds);
+	FD_SET(seq->fd, &rfds);
+	if (select(seq->fd + 1, &rfds, NULL, NULL, timeout) < 0)
+		return -errno;
+	if (FD_ISSET(seq->fd, &rfds))
+		return snd_seq_event_read_buffer(seq);
+	return seq->cells;
+}
+
+/*
+ * check events in input queue
+ */
+int snd_seq_event_input_pending(snd_seq_t *seq, int fetch_sequencer)
+{
+	if (snd_seq_input_cell_available(seq))
+		return seq->cells;
+
+	if (fetch_sequencer) {
+		struct timeval tv;
+		tv.tv_sec = 0;
+		tv.tv_usec = 0;
+		return snd_seq_event_input_feed(seq, &tv);
+	}
+	return seq->cells;
+}
+
+/* search the first matched event in input buffer and retrieve it */
+static int snd_seq_event_retrieve(snd_seq_t *seq, snd_seq_event_t **ev, int type)
+{
+	snd_seq_cell_t *c, *prev;
+
+	prev = NULL;
+	*ev = NULL;
+	for (c = seq->head; c; prev = c, c = c->next) {
+		if (c->ev.type == type) {
+			*ev = snd_seq_create_event();
+			if (*ev == NULL)
+				return -ENOMEM;
+			if (prev)
+				prev->next = c->next;
+			else
+				seq->head = c->next;
+			if (seq->tail == c)
+				seq->tail = prev;
+			seq->cells--;
+			memcpy(*ev, &c->ev, sizeof(snd_seq_event_t));
+			/* clear flag to avoid free copied data */
+			c->ev.flags &= ~SND_SEQ_EVENT_LENGTH_MASK;
+			snd_seq_free_cell(c);
+			return 0;
+		}
+	}
+	return 0;
+}
+
+
+/*
+ * retrieve an event from input FIFO with a specified type
+ */
+int snd_seq_event_input_selective(snd_seq_t *seq, snd_seq_event_t **ev, int type, int blocking)
+{
+	int err;
+
+	if (!seq)
+		return -EINVAL;
+
+	err = snd_seq_event_retrieve(seq, ev, type);
+	if (err < 0)
+		return err;
+	else if (*ev)
+		return seq->cells;
+
+	/* read from sequencer */
+	if (blocking) {
+		/* try until the event is found */
+		while (*ev == NULL) {
+			err = snd_seq_event_input_feed(seq, NULL);
+			if (err < 0)
+				return err;
+			err = snd_seq_event_retrieve(seq, ev, type);
+			if (err < 0)
+				return err;
+		}
+	} else {
+		/* try to read and check the new input buffer */
+		struct timeval tv;
+		tv.tv_sec = 0;
+		tv.tv_usec = 0;
+		err = snd_seq_event_input_feed(seq, &tv);
+		if (err < 0)
+			return err;
+		err = snd_seq_event_retrieve(seq, ev, type);
+		if (err < 0)
+			return err;
+	}
+
+	return seq->cells;
+}
+
+/*
+ * return the size of pending events
+ */
+int snd_seq_event_output_pending(snd_seq_t *seq)
+{
+	if (!seq)
+		return -EINVAL;
+	return seq->obufused;
+}
+
+/*
+ * flush output buffer to sequencer
+ */
 int snd_seq_flush_output(snd_seq_t *seq)
 {
 	int result;
 
 	if (!seq)
 		return -EINVAL;
-	if (seq->obufused <= 0)
-		return 0;
-	result = write(seq->fd, seq->obuf, seq->obufused);
-	if (result < 0)
-		return -errno;
-
-	if (result < seq->obufused)
-		memmove(seq->obuf, seq->obuf + result, seq->obufused - result);
-	seq->obufused -= result;
+	while (seq->obufused > 0) {
+		result = write(seq->fd, seq->obuf, seq->obufused);
+		if (result < 0)
+			return result;
+		if (result < seq->obufused)
+			memmove(seq->obuf, seq->obuf + result, seq->obufused - result);
+		seq->obufused -= result;
+	}
 	return seq->obufused;
 }
 
+
+/*
+ * extract the first event in output buffer
+ * if ev_res is NULL, just remove the event.
+ */
+int snd_seq_extract_output(snd_seq_t *seq, snd_seq_event_t **ev_res)
+{
+	int len, olen, err;
+	snd_seq_event_t *ev;
+
+	if (!seq)
+		return -EINVAL;
+	if (ev_res)
+		*ev_res = NULL;
+	if ((olen = seq->obufused) <= 0)
+		return -ENOENT;
+	if (ev_res) {
+		/* extract the event */
+		char *buf;
+		ev = snd_seq_create_event();
+		if (ev == NULL)
+			return -ENOMEM;
+		buf = seq->obuf;
+		len = olen;
+		err = snd_seq_decode_event(&buf, &len, ev);
+		if (err < 0) {
+			snd_seq_free_event(ev);
+			return err;
+		}
+		*ev_res = ev;
+	} else {
+		/* remove the event */
+		ev = (snd_seq_event_t*)seq->obuf;
+		len = olen - snd_seq_event_length(ev);
+	}
+	if (len > 0)
+		memmove(seq->obuf, seq->obuf + (olen - len), len);
+	seq->obufused = len;
+	return 0;
+}
+
+
+/*
+ * clear output buffer
+ */
+int snd_seq_drain_output_buffer(snd_seq_t *seq)
+{
+	if (!seq)
+		return -EINVAL;
+	seq->obufused = 0;
+	return 0;
+}
+
+/*
+ * clear input buffer
+ */
+int snd_seq_drain_input_buffer(snd_seq_t *seq)
+{
+	if (!seq)
+		return -EINVAL;
+	while (snd_seq_input_cell_available(seq) > 0)
+		snd_seq_free_cell(snd_seq_input_cell_out(seq));
+	return 0;
+}
+
+/*
+ * clear output buffer and remove events in sequencer queue
+ */
 int snd_seq_drain_output(snd_seq_t *seq)
 {
 	snd_seq_remove_events_t rminfo;
@@ -667,8 +882,23 @@ int snd_seq_drain_output(snd_seq_t *seq)
 	memset(&rminfo, 0, sizeof(rminfo));
 	rminfo.output = 1;
 
-	snd_seq_remove_events(seq, &rminfo);
-	return 0;
+	return snd_seq_remove_events(seq, &rminfo);
+}
+
+/*
+ * clear input buffer and and remove events in sequencer queue
+ */
+int snd_seq_drain_input(snd_seq_t *seq)
+{
+	snd_seq_remove_events_t rminfo;
+
+	if (!seq)
+		return -EINVAL;
+
+	memset(&rminfo, 0, sizeof(rminfo));
+	rminfo.input = 1;
+
+	return snd_seq_remove_events(seq, &rminfo);
 }
 
 /* compare timestamp between events */
@@ -703,7 +933,10 @@ static int remove_match(snd_seq_remove_events_t *info,
 			return 0;
 	}
 	if (info->remove_mode & SND_SEQ_REMOVE_DEST_CHANNEL) {
-		if (ev->dest.channel != info->dest.channel)
+		if (! snd_seq_ev_is_channel_type(ev))
+			return 0;
+		/* data.note.channel and data.control.channel are identical */
+		if (ev->data.note.channel != info->channel)
 			return 0;
 	}
 	if (info->remove_mode & SND_SEQ_REMOVE_TIME_AFTER) {
@@ -754,7 +987,9 @@ int snd_seq_remove_events(snd_seq_t *seq, snd_seq_remove_events_t *rmp)
 		 * First deal with any events that are still buffered
 		 * in the library.
 		 */
-		/* Input not implemented */
+		if (rmp->remove_mode == 0)
+			snd_seq_drain_input_buffer(seq);
+		/* other modes are not supported yet */
 	}
 
 	if (rmp->output) {
@@ -763,8 +998,8 @@ int snd_seq_remove_events(snd_seq_t *seq, snd_seq_remove_events_t *rmp)
 		 * in the library.
 		 */
 		 if (rmp->remove_mode == 0) {
-			/* The simple case - remove all */
-			seq->obufused = 0;
+			 /* The simple case - remove all */
+			 snd_seq_drain_output_buffer(seq);
 		} else {
 			char *ep;
 			int  len;
@@ -777,10 +1012,9 @@ int snd_seq_remove_events(snd_seq_t *seq, snd_seq_remove_events_t *rmp)
 				len = snd_seq_event_length(ev);
 
 				if (remove_match(rmp, ev)) {
-
 					/* Remove event */
-					memcpy(ep, ep + len, len);
 					seq->obufused -= len;
+					memmove(ep, ep + len, seq->obufused - (ep - seq->obuf));
 				} else {
 					ep += len;
 				}
@@ -793,14 +1027,6 @@ int snd_seq_remove_events(snd_seq_t *seq, snd_seq_remove_events_t *rmp)
 
 	return 0;
 }
-int snd_seq_drain_input(snd_seq_t *seq)
-{
-	if (!seq)
-		return -EINVAL;
-	while (snd_seq_input_cell_available(seq))
-		snd_seq_free_cell(snd_seq_input_cell_out(seq));
-	return 0;
-}
 
 int snd_seq_get_client_pool(snd_seq_t *seq, snd_seq_client_pool_t *info)
 {
@@ -826,7 +1052,6 @@ int snd_seq_query_next_client(snd_seq_t *seq, snd_seq_client_info_t *info)
 {
 	if (!seq || !info)
 		return -EINVAL;
-	info->client = seq->client;
 	if (ioctl(seq->fd, SND_SEQ_IOCTL_QUERY_NEXT_CLIENT, info) < 0)
 		return -errno;
 	return 0;
@@ -836,7 +1061,6 @@ int snd_seq_query_next_port(snd_seq_t *seq, snd_seq_port_info_t *info)
 {
 	if (!seq || !info)
 		return -EINVAL;
-	info->client = seq->client;
 	if (ioctl(seq->fd, SND_SEQ_IOCTL_QUERY_NEXT_PORT, info) < 0)
 		return -errno;
 	return 0;
@@ -861,4 +1085,3 @@ int snd_seq_get_bit(int nr, void *array)
 	return ((((unsigned int *)array)[nr >> 5]) & (1UL << (nr & 31))) ? 1 : 0;
 }
 
-
